@@ -1,7 +1,9 @@
+import json
 import os
-import re
-import sys
 import pytest
+import re
+import subprocess
+import sys
 
 from threadpoolctl import threadpool_limits, threadpool_info, _ThreadpoolInfo
 from threadpoolctl import _ALL_PREFIXES, _ALL_USER_APIS
@@ -9,6 +11,7 @@ from threadpoolctl import _ALL_PREFIXES, _ALL_USER_APIS
 from .utils import cython_extensions_compiled
 from .utils import libopenblas_paths
 from .utils import scipy
+from .utils import threadpool_info_from_subprocess
 
 
 def is_old_openblas(module):
@@ -167,7 +170,8 @@ def test_threadpool_limits_bad_input():
 def test_openmp_limit_num_threads(num_threads):
     # checks that OpenMP effectively uses the number of threads requested by
     # the context manager
-    from ._openmp_test_helper import check_openmp_num_threads
+    import tests._openmp_test_helper.openmp_helpers_inner as omp_inner
+    check_openmp_num_threads = omp_inner.check_openmp_num_threads
 
     old_num_threads = check_openmp_num_threads(100)
 
@@ -177,37 +181,40 @@ def test_openmp_limit_num_threads(num_threads):
 
 
 @pytest.mark.skipif(not cython_extensions_compiled,
-                    reason='Requires cython extensions to be compiled')
-@pytest.mark.parametrize('nthreads_outer', [None, 1, 2, 4])
+                    reason="Requires cython extensions to be compiled")
+@pytest.mark.parametrize("nthreads_outer", [None, 1, 2, 4])
 def test_openmp_nesting(nthreads_outer):
     # checks that OpenMP effectively uses the number of threads requested by
     # the context manager when nested in an outer OpenMP loop.
-    from ._openmp_test_helper import check_nested_openmp_loops
-    from ._openmp_test_helper import get_inner_compiler
-    from ._openmp_test_helper import get_outer_compiler
+    import tests._openmp_test_helper.openmp_helpers_outer as omp_outer
+    check_nested_openmp_loops = omp_outer.check_nested_openmp_loops
 
-    inner_cc = get_inner_compiler()
-    outer_cc = get_outer_compiler()
+    # Find which OpenMP lib is used at runtime for inner loop
+    inner_info = threadpool_info_from_subprocess(
+        "tests._openmp_test_helper.openmp_helpers_inner")
+    assert len(inner_info) == 1
+    inner_omp = inner_info[0]["prefix"]
+
+    # Find which OpenMP lib is used at runtime for outer loop
+    outer_info = threadpool_info_from_subprocess(
+        "tests._openmp_test_helper.openmp_helpers_outer")
+    if len(outer_info) == 1:
+        # Only 1 openmp loaded. It has to be this one.
+        outer_omp = outer_info[0]["prefix"]
+    else:
+        # There are 2 openmp, the one from inner and the one from outer.
+        assert len(outer_info) == 2
+        # We already know the one from inner. It has to be the other one.
+        prefixes = {module["prefix"] for module in outer_info}
+        outer_omp = prefixes - {inner_omp}
 
     outer_num_threads, inner_num_threads = check_nested_openmp_loops(10)
-
     original_info = _threadpool_info()
-    openmp_info = original_info.get_modules("user_api", "openmp")
 
-    if "gcc" in (inner_cc, outer_cc):
-        assert original_info.get_modules("prefix", "libgomp")
-
-    if "clang" in (inner_cc, outer_cc):
-        assert original_info.get_modules("prefix", "libomp")
-
-    if inner_cc == outer_cc:
-        # The openmp runtime should be shared by default, meaning that
-        # the inner loop should automatically be run serially by the OpenMP
-        # runtime.
+    if inner_omp == outer_omp:
+        # The OpenMP runtime should be shared by default, meaning that the
+        # inner loop should automatically be run serially by the OpenMP runtime
         assert inner_num_threads == 1
-    else:
-        # There should be at least 2 OpenMP runtime detected.
-        assert len(openmp_info) >= 2
 
     with threadpool_limits(limits=1) as threadpoolctx:
         max_threads = threadpoolctx.get_original_num_threads()["openmp"]
@@ -227,8 +234,8 @@ def test_openmp_nesting(nthreads_outer):
     assert outer_num_threads == nthreads
 
     # The number of threads available in the inner loop should have been
-    # set to 1 so avoid oversubscription and preserve performance:
-    if inner_cc != outer_cc:
+    # set to 1 to avoid oversubscription and preserve performance:
+    if inner_omp != outer_omp:
         if inner_num_threads != 1:
             # XXX: this does not always work when nesting independent openmp
             # implementations. See: https://github.com/jeremiedbb/Nested_OpenMP
@@ -261,13 +268,16 @@ def test_multiple_shipped_openblas():
 
 
 @pytest.mark.skipif(scipy is None, reason="requires scipy")
+@pytest.mark.skipif(not cython_extensions_compiled,
+                    reason='Requires cython extensions to be compiled')
 @pytest.mark.parametrize("nthreads_outer", [None, 1, 2, 4])
 def test_nested_prange_blas(nthreads_outer):
     # Check that the BLAS linked to scipy effectively uses the number of
     # threads requested by the context manager when nested in an outer OpenMP
     # loop.
     import numpy as np
-    from ._openmp_test_helper import check_nested_prange_blas
+    import tests._openmp_test_helper.nested_prange_blas as prange_blas
+    check_nested_prange_blas = prange_blas.check_nested_prange_blas
 
     original_info = _threadpool_info()
 
@@ -368,18 +378,18 @@ def test_blis_threading_layer():
                     reason='Requires cython extensions to be compiled')
 def test_libomp_libiomp_warning(recwarn):
     # Trigger the import of a potentially clang-compiled extension:
-    from ._openmp_test_helper import check_nested_openmp_loops  # noqa
+    import tests._openmp_test_helper.openmp_helpers_outer  # noqa
 
     # Trigger the import of numpy to potentially import Intel OpenMP via MKL
-    import numpy.linalg  # noqa
+    pytest.importorskip("numpy.linalg")
 
     # Check that a warning is raised when both libomp and libiomp are loaded
     # It should happen in one CI job (pylatest_conda_mkl_clang_gcc).
     info = _threadpool_info()
     prefixes = [module.prefix for module in info]
 
-    if not ("libomp" in prefixes and "libiomp" in prefixes
-            and sys.platform == "linux"):
+    if not ("libomp" in prefixes and "libiomp" in prefixes and
+            sys.platform == "linux"):
         pytest.skip("Requires both libomp and libiomp loaded, on Linux")
 
     assert len(recwarn) == 1
@@ -388,3 +398,68 @@ def test_libomp_libiomp_warning(recwarn):
     assert "Found Intel" in str(wm.message)
     assert "LLVM" in str(wm.message)
     assert "multiple_openmp.md" in str(wm.message)
+
+
+def test_command_line_empty():
+    output = subprocess.check_output(
+        (sys.executable + " -m threadpoolctl").split())
+    assert json.loads(output.decode("utf-8")) == []
+
+
+def test_command_line_command_flag():
+    pytest.importorskip("numpy")
+    output = subprocess.check_output(
+        [sys.executable, "-m", "threadpoolctl", "-c", "import numpy"])
+    cli_info = json.loads(output.decode("utf-8"))
+
+    this_process_info = threadpool_info()
+    for module in cli_info:
+        assert module in this_process_info
+
+
+@pytest.mark.skipif(sys.version_info < (3, 7),
+                    reason="need recent subprocess.run options")
+def test_command_line_import_flag():
+    result = subprocess.run([
+        sys.executable, "-m", "threadpoolctl", "-i",
+        "numpy",
+        "scipy.linalg",
+        "invalid_package",
+        "numpy.invalid_sumodule",
+    ], capture_output=True, check=True, encoding="utf-8")
+    cli_info = json.loads(result.stdout)
+
+    this_process_info = threadpool_info()
+    for module in cli_info:
+        assert module in this_process_info
+
+    warnings = [w.strip() for w in result.stderr.splitlines()]
+    assert "WARNING: could not import invalid_package" in warnings
+    assert "WARNING: could not import numpy.invalid_sumodule" in warnings
+    if scipy is None:
+        assert "WARNING: could not import scipy.linalg" in warnings
+    else:
+        assert "WARNING: could not import scipy.linalg" not in warnings
+
+
+def test_architecture():
+    expected_openblas_architectures = (
+        # XXX: add more as needed by CI or developer laptops
+        "armv8",
+        "Haswell",
+        "SkylakeX",
+        "Sandybridge",
+    )
+    expected_blis_architectures = (
+        # XXX: add more as needed by CI or developer laptops
+        "skx",
+        "haswell",
+    )
+    for module in threadpool_info():
+        if module["internal_api"] == "openblas":
+            assert module["architecture"] in expected_openblas_architectures
+        elif module["internal_api"] == "blis":
+            assert module["architecture"] in expected_blis_architectures
+        else:
+            # Not supported for other modules
+            assert "architecture" not in module
